# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_descend_and_pass_through_visitor.py
# Do NOT edit or append.


"""
Test jointly :py:method:`aas_core3.types.Class.descend` and
:py:method:`aas_core3.types.PassThroughVisitor`.
"""


# pylint: disable=missing-docstring


from typing import (
    List,
    Sequence,
)
import unittest

import aas_core3.types as aas_types

import tests.common
import tests.common_jsonization


class _TracingVisitor(aas_types.PassThroughVisitor):
    """Visit the instances and trace them."""

    def __init__(self) -> None:
        """Initialize with an empty log."""
        self._log = []  # type: List[str]

    @property
    def log(self) -> Sequence[str]:
        """Get the tracing log."""
        return self._log

    def visit(self, that: aas_types.Class) -> None:
        self._log.append(tests.common.trace(that))
        super().visit(that)


def assert_tracing_logs_from_descend_and_visitor_are_the_same(
    that: aas_types.Class, test_case: unittest.TestCase
) -> None:
    """
    Check that the tracing logs are the same when :paramref:`that` instance
    is visited and when :paramref:`that` is ran through
    :py:method:`aas_types.Class.descend`.

    :param that: instance to be iterated over
    :param test_case: in which this assertion runs
    :raise: :py:class:`AssertionError` if the logs differ
    """
    log_from_descend = [tests.common.trace(something) for something in that.descend()]

    visitor = _TracingVisitor()
    visitor.visit(that)
    log_from_visitor = visitor.log

    test_case.assertGreater(len(log_from_visitor), 0)
    test_case.assertEqual(tests.common.trace(that), log_from_visitor[0])

    # noinspection PyTypeChecker
    test_case.assertListEqual(log_from_descend, log_from_visitor[1:])  # type: ignore


class Test_Extension(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_extension()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Extension"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_extension()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_AdministrativeInformation(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_administrative_information()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "AdministrativeInformation"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_administrative_information()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Qualifier(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_qualifier()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Qualifier"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_qualifier()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_AssetAdministrationShell(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_administration_shell()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "AssetAdministrationShell"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_administration_shell()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_AssetInformation(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_information()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "AssetInformation"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_information()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Resource(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_resource()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Resource"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_resource()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_SpecificAssetID(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_specific_asset_id()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "SpecificAssetId"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_specific_asset_id()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Submodel(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Submodel"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_RelationshipElement(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_relationship_element()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "RelationshipElement"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_relationship_element()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_SubmodelElementList(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_list()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "SubmodelElementList"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_list()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_SubmodelElementCollection(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_collection()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "SubmodelElementCollection"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_collection()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Property(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_property()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Property"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_property()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_MultiLanguageProperty(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_multi_language_property()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "MultiLanguageProperty"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_multi_language_property()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Range(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_range()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Range"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_range()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_ReferenceElement(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference_element()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "ReferenceElement"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference_element()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Blob(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_blob()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Blob"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_blob()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_File(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_file()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "File"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_file()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_AnnotatedRelationshipElement(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "AnnotatedRelationshipElement"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Entity(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_entity()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Entity"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_entity()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_EventPayload(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_event_payload()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "EventPayload"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_event_payload()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_BasicEventElement(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_basic_event_element()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "BasicEventElement"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_basic_event_element()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Operation(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Operation"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_OperationVariable(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation_variable()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "OperationVariable"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation_variable()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Capability(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_capability()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Capability"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_capability()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_ConceptDescription(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_concept_description()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "ConceptDescription"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_concept_description()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Reference(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Reference"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Key(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_key()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Key"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_key()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_LangStringNameType(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_name_type()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "LangStringNameType"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_name_type()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_LangStringTextType(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_text_type()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "LangStringTextType"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_text_type()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_Environment(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_environment()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "Environment"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_environment()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_EmbeddedDataSpecification(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_embedded_data_specification()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "EmbeddedDataSpecification"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_embedded_data_specification()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_LevelType(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_level_type()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "LevelType"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_level_type()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_ValueReferencePair(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_reference_pair()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "ValueReferencePair"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_reference_pair()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_ValueList(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_list()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "ValueList"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_list()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_LangStringPreferredNameTypeIEC61360(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_preferred_name_type_iec_61360()
        )
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "LangStringPreferredNameTypeIec61360"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_preferred_name_type_iec_61360()
        )
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_LangStringShortNameTypeIEC61360(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_short_name_type_iec_61360()
        )
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "LangStringShortNameTypeIec61360"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_short_name_type_iec_61360()
        )
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_LangStringDefinitionTypeIEC61360(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_definition_type_iec_61360()
        )
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "LangStringDefinitionTypeIec61360"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_definition_type_iec_61360()
        )
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


class Test_DataSpecificationIEC61360(unittest.TestCase):
    def test_descend_against_recorded_trace_log(self) -> None:
        instance = tests.common_jsonization.load_maximal_data_specification_iec_61360()
        expected_path = (
            tests.common.TEST_DATA_DIR
            / "descend_and_pass_through_visitor"
            / "DataSpecificationIec61360"
            / "maximal.json.trace"
        )

        log = [tests.common.trace(something) for something in instance.descend()]
        got_text = tests.common.trace_log_as_text_file_content(log)
        tests.common.record_or_check(expected_path, got_text)

    def test_descend_against_visitor(self) -> None:
        instance = tests.common_jsonization.load_maximal_data_specification_iec_61360()
        assert_tracing_logs_from_descend_and_visitor_are_the_same(instance, self)


if __name__ == "__main__":
    unittest.main()


# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_descend_and_pass_through_visitor.py
# Do NOT edit or append.



# pylint: disable=missing-docstring

# NOTE (mristin):
# We explicitly check that dictionaries are not mistaken for array-like objects during
# the de-serialization.
#
# This is a regression test for the following issue:
# https://github.com/aas-core-works/aas-core3.0-python/issues/31

import unittest
from typing import Optional

import aas_core3.jsonization as aasjsonization


class TestIsArrayLike(unittest.TestCase):
    def test_example(self) -> None:
        jsonable = {"submodels": {}}  # type: aasjsonization.Jsonable

        got_exception = None  # type: Optional[aasjsonization.DeserializationException]

        try:
            _ = aasjsonization.environment_from_jsonable(jsonable)
        except aasjsonization.DeserializationException as exception:
            got_exception = exception

        assert got_exception is not None
        self.assertEqual(
            "Expected something array-like, but got: <class 'dict'>", str(got_exception)
        )


if __name__ == "__main__":
    unittest.main()



# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_xmlization_of_concrete_classes_outside_container.py
# Do NOT edit or append.


"""Test de/serialization from XML of concrete classes outside a container."""


# pylint: disable=missing-docstring


import unittest
import xml.etree.ElementTree as ET

import aas_core3.xmlization as aas_xmlization
import aas_core3.verification as aas_verification

import tests.common
import tests.common_jsonization
import tests.common_xmlization


class Test_Extension(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Extension` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_extension()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.extension_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_AdministrativeInformation(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.AdministrativeInformation` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_administrative_information()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.administrative_information_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Qualifier(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Qualifier` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_qualifier()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.qualifier_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_AssetAdministrationShell(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.AssetAdministrationShell` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_administration_shell()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.asset_administration_shell_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_AssetInformation(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.AssetInformation` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_information()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.asset_information_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Resource(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Resource` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_resource()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.resource_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_SpecificAssetID(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.SpecificAssetID` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_specific_asset_id()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.specific_asset_id_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Submodel(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Submodel` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.submodel_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_RelationshipElement(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.RelationshipElement` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_relationship_element()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.relationship_element_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_SubmodelElementList(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.SubmodelElementList` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_list()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.submodel_element_list_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_SubmodelElementCollection(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.SubmodelElementCollection` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_collection()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.submodel_element_collection_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Property(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Property` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_property()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.property_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_MultiLanguageProperty(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.MultiLanguageProperty` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_multi_language_property()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.multi_language_property_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Range(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Range` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_range()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.range_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_ReferenceElement(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.ReferenceElement` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference_element()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.reference_element_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Blob(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Blob` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_blob()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.blob_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_File(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.File` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_file()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.file_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_AnnotatedRelationshipElement(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.AnnotatedRelationshipElement` outside a container.
    """

    def test_ok(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.annotated_relationship_element_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Entity(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Entity` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_entity()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.entity_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_BasicEventElement(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.BasicEventElement` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_basic_event_element()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.basic_event_element_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Operation(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Operation` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.operation_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_OperationVariable(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.OperationVariable` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation_variable()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.operation_variable_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Capability(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Capability` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_capability()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.capability_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_ConceptDescription(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.ConceptDescription` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_concept_description()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.concept_description_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Reference(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Reference` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.reference_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_Key(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.Key` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_key()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.key_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_LangStringNameType(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.LangStringNameType` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_name_type()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.lang_string_name_type_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_LangStringTextType(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.LangStringTextType` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_text_type()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.lang_string_text_type_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_EmbeddedDataSpecification(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.EmbeddedDataSpecification` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_embedded_data_specification()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.embedded_data_specification_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_LevelType(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.LevelType` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_level_type()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.level_type_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_ValueReferencePair(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.ValueReferencePair` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_reference_pair()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.value_reference_pair_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_ValueList(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.ValueList` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_list()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.value_list_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_LangStringPreferredNameTypeIEC61360(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.LangStringPreferredNameTypeIEC61360` outside a container.
    """

    def test_ok(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_preferred_name_type_iec_61360()
        )
        text = aas_xmlization.to_str(instance)

        another_instance = (
            aas_xmlization.lang_string_preferred_name_type_iec_61360_from_str(text)
        )

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_LangStringShortNameTypeIEC61360(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.LangStringShortNameTypeIEC61360` outside a container.
    """

    def test_ok(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_short_name_type_iec_61360()
        )
        text = aas_xmlization.to_str(instance)

        another_instance = (
            aas_xmlization.lang_string_short_name_type_iec_61360_from_str(text)
        )

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_LangStringDefinitionTypeIEC61360(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.LangStringDefinitionTypeIEC61360` outside a container.
    """

    def test_ok(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_definition_type_iec_61360()
        )
        text = aas_xmlization.to_str(instance)

        another_instance = (
            aas_xmlization.lang_string_definition_type_iec_61360_from_str(text)
        )

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


class Test_DataSpecificationIEC61360(unittest.TestCase):
    """
    Test XML de/serialization of the concrete class
    :py:class:`aas_core3.types.DataSpecificationIEC61360` outside a container.
    """

    def test_ok(self) -> None:
        instance = tests.common_jsonization.load_maximal_data_specification_iec_61360()
        text = aas_xmlization.to_str(instance)

        another_instance = aas_xmlization.data_specification_iec_61360_from_str(text)

        errors = list(aas_verification.verify(another_instance))
        self.assertListEqual([], errors)

        et_instance = ET.fromstring(text)
        tests.common_xmlization.remove_redundant_whitespace(et_instance)

        et_another_instance = ET.fromstring(aas_xmlization.to_str(another_instance))
        tests.common_xmlization.remove_redundant_whitespace(et_another_instance)

        tests.common_xmlization.assert_elements_equal(et_instance, et_another_instance)


if __name__ == "__main__":
    unittest.main()


# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_xmlization_of_concrete_classes_outside_container.py
# Do NOT edit or append.



"""De-serialize enumerations from string representations."""

# This code has been automatically generated by aas-core-codegen.
# Do NOT edit or append.


from typing import (
    Mapping,
    Optional,
)

import aas_core3.types as aas_types


_MODELLING_KIND_FROM_STR: Mapping[str, aas_types.ModellingKind] = {
    "Template": aas_types.ModellingKind.TEMPLATE,
    "Instance": aas_types.ModellingKind.INSTANCE,
}


def modelling_kind_from_str(text: str) -> Optional[aas_types.ModellingKind]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.ModellingKind`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.ModellingKind`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.ModellingKind`
        or ``None``, if :paramref:`text` invalid.
    """
    return _MODELLING_KIND_FROM_STR.get(text, None)


_QUALIFIER_KIND_FROM_STR: Mapping[str, aas_types.QualifierKind] = {
    "ValueQualifier": aas_types.QualifierKind.VALUE_QUALIFIER,
    "ConceptQualifier": aas_types.QualifierKind.CONCEPT_QUALIFIER,
    "TemplateQualifier": aas_types.QualifierKind.TEMPLATE_QUALIFIER,
}


def qualifier_kind_from_str(text: str) -> Optional[aas_types.QualifierKind]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.QualifierKind`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.QualifierKind`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.QualifierKind`
        or ``None``, if :paramref:`text` invalid.
    """
    return _QUALIFIER_KIND_FROM_STR.get(text, None)


_ASSET_KIND_FROM_STR: Mapping[str, aas_types.AssetKind] = {
    "Type": aas_types.AssetKind.TYPE,
    "Instance": aas_types.AssetKind.INSTANCE,
    "NotApplicable": aas_types.AssetKind.NOT_APPLICABLE,
}


def asset_kind_from_str(text: str) -> Optional[aas_types.AssetKind]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.AssetKind`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.AssetKind`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.AssetKind`
        or ``None``, if :paramref:`text` invalid.
    """
    return _ASSET_KIND_FROM_STR.get(text, None)


_AAS_SUBMODEL_ELEMENTS_FROM_STR: Mapping[str, aas_types.AASSubmodelElements] = {
    "AnnotatedRelationshipElement": aas_types.AASSubmodelElements.ANNOTATED_RELATIONSHIP_ELEMENT,
    "BasicEventElement": aas_types.AASSubmodelElements.BASIC_EVENT_ELEMENT,
    "Blob": aas_types.AASSubmodelElements.BLOB,
    "Capability": aas_types.AASSubmodelElements.CAPABILITY,
    "DataElement": aas_types.AASSubmodelElements.DATA_ELEMENT,
    "Entity": aas_types.AASSubmodelElements.ENTITY,
    "EventElement": aas_types.AASSubmodelElements.EVENT_ELEMENT,
    "File": aas_types.AASSubmodelElements.FILE,
    "MultiLanguageProperty": aas_types.AASSubmodelElements.MULTI_LANGUAGE_PROPERTY,
    "Operation": aas_types.AASSubmodelElements.OPERATION,
    "Property": aas_types.AASSubmodelElements.PROPERTY,
    "Range": aas_types.AASSubmodelElements.RANGE,
    "ReferenceElement": aas_types.AASSubmodelElements.REFERENCE_ELEMENT,
    "RelationshipElement": aas_types.AASSubmodelElements.RELATIONSHIP_ELEMENT,
    "SubmodelElement": aas_types.AASSubmodelElements.SUBMODEL_ELEMENT,
    "SubmodelElementList": aas_types.AASSubmodelElements.SUBMODEL_ELEMENT_LIST,
    "SubmodelElementCollection": aas_types.AASSubmodelElements.SUBMODEL_ELEMENT_COLLECTION,
}


def aas_submodel_elements_from_str(
    text: str,
) -> Optional[aas_types.AASSubmodelElements]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.AASSubmodelElements`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.AASSubmodelElements`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.AASSubmodelElements`
        or ``None``, if :paramref:`text` invalid.
    """
    return _AAS_SUBMODEL_ELEMENTS_FROM_STR.get(text, None)


_ENTITY_TYPE_FROM_STR: Mapping[str, aas_types.EntityType] = {
    "CoManagedEntity": aas_types.EntityType.CO_MANAGED_ENTITY,
    "SelfManagedEntity": aas_types.EntityType.SELF_MANAGED_ENTITY,
}


def entity_type_from_str(text: str) -> Optional[aas_types.EntityType]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.EntityType`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.EntityType`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.EntityType`
        or ``None``, if :paramref:`text` invalid.
    """
    return _ENTITY_TYPE_FROM_STR.get(text, None)


_DIRECTION_FROM_STR: Mapping[str, aas_types.Direction] = {
    "input": aas_types.Direction.INPUT,
    "output": aas_types.Direction.OUTPUT,
}


def direction_from_str(text: str) -> Optional[aas_types.Direction]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.Direction`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.Direction`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.Direction`
        or ``None``, if :paramref:`text` invalid.
    """
    return _DIRECTION_FROM_STR.get(text, None)


_STATE_OF_EVENT_FROM_STR: Mapping[str, aas_types.StateOfEvent] = {
    "on": aas_types.StateOfEvent.ON,
    "off": aas_types.StateOfEvent.OFF,
}


def state_of_event_from_str(text: str) -> Optional[aas_types.StateOfEvent]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.StateOfEvent`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.StateOfEvent`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.StateOfEvent`
        or ``None``, if :paramref:`text` invalid.
    """
    return _STATE_OF_EVENT_FROM_STR.get(text, None)


_REFERENCE_TYPES_FROM_STR: Mapping[str, aas_types.ReferenceTypes] = {
    "ExternalReference": aas_types.ReferenceTypes.EXTERNAL_REFERENCE,
    "ModelReference": aas_types.ReferenceTypes.MODEL_REFERENCE,
}


def reference_types_from_str(text: str) -> Optional[aas_types.ReferenceTypes]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.ReferenceTypes`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.ReferenceTypes`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.ReferenceTypes`
        or ``None``, if :paramref:`text` invalid.
    """
    return _REFERENCE_TYPES_FROM_STR.get(text, None)


_KEY_TYPES_FROM_STR: Mapping[str, aas_types.KeyTypes] = {
    "AnnotatedRelationshipElement": aas_types.KeyTypes.ANNOTATED_RELATIONSHIP_ELEMENT,
    "AssetAdministrationShell": aas_types.KeyTypes.ASSET_ADMINISTRATION_SHELL,
    "BasicEventElement": aas_types.KeyTypes.BASIC_EVENT_ELEMENT,
    "Blob": aas_types.KeyTypes.BLOB,
    "Capability": aas_types.KeyTypes.CAPABILITY,
    "ConceptDescription": aas_types.KeyTypes.CONCEPT_DESCRIPTION,
    "DataElement": aas_types.KeyTypes.DATA_ELEMENT,
    "Entity": aas_types.KeyTypes.ENTITY,
    "EventElement": aas_types.KeyTypes.EVENT_ELEMENT,
    "File": aas_types.KeyTypes.FILE,
    "FragmentReference": aas_types.KeyTypes.FRAGMENT_REFERENCE,
    "GlobalReference": aas_types.KeyTypes.GLOBAL_REFERENCE,
    "Identifiable": aas_types.KeyTypes.IDENTIFIABLE,
    "MultiLanguageProperty": aas_types.KeyTypes.MULTI_LANGUAGE_PROPERTY,
    "Operation": aas_types.KeyTypes.OPERATION,
    "Property": aas_types.KeyTypes.PROPERTY,
    "Range": aas_types.KeyTypes.RANGE,
    "Referable": aas_types.KeyTypes.REFERABLE,
    "ReferenceElement": aas_types.KeyTypes.REFERENCE_ELEMENT,
    "RelationshipElement": aas_types.KeyTypes.RELATIONSHIP_ELEMENT,
    "Submodel": aas_types.KeyTypes.SUBMODEL,
    "SubmodelElement": aas_types.KeyTypes.SUBMODEL_ELEMENT,
    "SubmodelElementCollection": aas_types.KeyTypes.SUBMODEL_ELEMENT_COLLECTION,
    "SubmodelElementList": aas_types.KeyTypes.SUBMODEL_ELEMENT_LIST,
}


def key_types_from_str(text: str) -> Optional[aas_types.KeyTypes]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.KeyTypes`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.KeyTypes`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.KeyTypes`
        or ``None``, if :paramref:`text` invalid.
    """
    return _KEY_TYPES_FROM_STR.get(text, None)


_DATA_TYPE_DEF_XSD_FROM_STR: Mapping[str, aas_types.DataTypeDefXSD] = {
    "xs:anyURI": aas_types.DataTypeDefXSD.ANY_URI,
    "xs:base64Binary": aas_types.DataTypeDefXSD.BASE_64_BINARY,
    "xs:boolean": aas_types.DataTypeDefXSD.BOOLEAN,
    "xs:byte": aas_types.DataTypeDefXSD.BYTE,
    "xs:date": aas_types.DataTypeDefXSD.DATE,
    "xs:dateTime": aas_types.DataTypeDefXSD.DATE_TIME,
    "xs:decimal": aas_types.DataTypeDefXSD.DECIMAL,
    "xs:double": aas_types.DataTypeDefXSD.DOUBLE,
    "xs:duration": aas_types.DataTypeDefXSD.DURATION,
    "xs:float": aas_types.DataTypeDefXSD.FLOAT,
    "xs:gDay": aas_types.DataTypeDefXSD.G_DAY,
    "xs:gMonth": aas_types.DataTypeDefXSD.G_MONTH,
    "xs:gMonthDay": aas_types.DataTypeDefXSD.G_MONTH_DAY,
    "xs:gYear": aas_types.DataTypeDefXSD.G_YEAR,
    "xs:gYearMonth": aas_types.DataTypeDefXSD.G_YEAR_MONTH,
    "xs:hexBinary": aas_types.DataTypeDefXSD.HEX_BINARY,
    "xs:int": aas_types.DataTypeDefXSD.INT,
    "xs:integer": aas_types.DataTypeDefXSD.INTEGER,
    "xs:long": aas_types.DataTypeDefXSD.LONG,
    "xs:negativeInteger": aas_types.DataTypeDefXSD.NEGATIVE_INTEGER,
    "xs:nonNegativeInteger": aas_types.DataTypeDefXSD.NON_NEGATIVE_INTEGER,
    "xs:nonPositiveInteger": aas_types.DataTypeDefXSD.NON_POSITIVE_INTEGER,
    "xs:positiveInteger": aas_types.DataTypeDefXSD.POSITIVE_INTEGER,
    "xs:short": aas_types.DataTypeDefXSD.SHORT,
    "xs:string": aas_types.DataTypeDefXSD.STRING,
    "xs:time": aas_types.DataTypeDefXSD.TIME,
    "xs:unsignedByte": aas_types.DataTypeDefXSD.UNSIGNED_BYTE,
    "xs:unsignedInt": aas_types.DataTypeDefXSD.UNSIGNED_INT,
    "xs:unsignedLong": aas_types.DataTypeDefXSD.UNSIGNED_LONG,
    "xs:unsignedShort": aas_types.DataTypeDefXSD.UNSIGNED_SHORT,
}


def data_type_def_xsd_from_str(text: str) -> Optional[aas_types.DataTypeDefXSD]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.DataTypeDefXSD`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.DataTypeDefXSD`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.DataTypeDefXSD`
        or ``None``, if :paramref:`text` invalid.
    """
    return _DATA_TYPE_DEF_XSD_FROM_STR.get(text, None)


_DATA_TYPE_IEC_61360_FROM_STR: Mapping[str, aas_types.DataTypeIEC61360] = {
    "DATE": aas_types.DataTypeIEC61360.DATE,
    "STRING": aas_types.DataTypeIEC61360.STRING,
    "STRING_TRANSLATABLE": aas_types.DataTypeIEC61360.STRING_TRANSLATABLE,
    "INTEGER_MEASURE": aas_types.DataTypeIEC61360.INTEGER_MEASURE,
    "INTEGER_COUNT": aas_types.DataTypeIEC61360.INTEGER_COUNT,
    "INTEGER_CURRENCY": aas_types.DataTypeIEC61360.INTEGER_CURRENCY,
    "REAL_MEASURE": aas_types.DataTypeIEC61360.REAL_MEASURE,
    "REAL_COUNT": aas_types.DataTypeIEC61360.REAL_COUNT,
    "REAL_CURRENCY": aas_types.DataTypeIEC61360.REAL_CURRENCY,
    "BOOLEAN": aas_types.DataTypeIEC61360.BOOLEAN,
    "IRI": aas_types.DataTypeIEC61360.IRI,
    "IRDI": aas_types.DataTypeIEC61360.IRDI,
    "RATIONAL": aas_types.DataTypeIEC61360.RATIONAL,
    "RATIONAL_MEASURE": aas_types.DataTypeIEC61360.RATIONAL_MEASURE,
    "TIME": aas_types.DataTypeIEC61360.TIME,
    "TIMESTAMP": aas_types.DataTypeIEC61360.TIMESTAMP,
    "FILE": aas_types.DataTypeIEC61360.FILE,
    "HTML": aas_types.DataTypeIEC61360.HTML,
    "BLOB": aas_types.DataTypeIEC61360.BLOB,
}


def data_type_iec_61360_from_str(text: str) -> Optional[aas_types.DataTypeIEC61360]:
    """
    Parse :paramref:`text` as string representation
    of :py:class:`aas_core3.DataTypeIEC61360`.

    If :paramref:`text` is not a valid string representation of a literal
    of :py:class:`aas_core3.DataTypeIEC61360`, return ``None``.

    :param text: to be parsed
    :return:
        the corresponding literal of :py:class:`aas_core3.DataTypeIEC61360`
        or ``None``, if :paramref:`text` invalid.
    """
    return _DATA_TYPE_IEC_61360_FROM_STR.get(text, None)


# This code has been automatically generated by aas-core-codegen.
# Do NOT edit or append.
